<?php
/**
 * @file
 * The Connection class
 */

namespace Drupal\ming\Ming;

/**
 * Class Connection
 *
 * @package Drupal\ming\Ming
 */
class Connection {

  /**
   * The Mongo connection
   *
   * Because this is a public object, we can call all the usual Mongo methods on
   * it, for example Connection::connection->listDBs
   *
   * @var \MongoClient
   */
  public $connection;

  /**
   * The Db
   *
   * @var \MongoDB
   */
  public $database;

  /**
   * Constructor.
   *
   * @param array $settings
   *   Connection settings.
   */
  public function __construct($settings) {
    $this->connect($settings);

    return $this;
  }

  /**
   * Connect to connection.
   *
   * @todo Add Replica Set and Socket support
   * @todo Should we support wildcard/random persistent connections?
   *
   * @param array $settings
   *   An array of database settings
   * @param array $options
   *   Other options to use for the connection (e.g. Replica Sets et al)
   *
   * @return \Drupal\ming\Ming\Connection
   *   A PHP Mongo class
   */
  public function connect($settings, $options = array()) {

    // Options.
    $default_options = array("persist" => "ming_" . $settings['connection_key']);
    $options = $default_options + $options;

    // Default to localhost if not provided.
    if (!isset($settings['mongo_host'])) {
      $settings['mongo_host'] = 'localhost';
    }

    // Build a connection string.
    $connection_string = '';
    if (isset($settings['mongo_user']) && !empty($settings['mongo_user'])) {
      $connection_string = "${settings['mongo_user']}:${$settings['mongo_pass']}@";
    }
    $connection_string .= $connection_string . $settings['mongo_host'];
    if (isset($settings['mongo_port'])) {
      $connection_string .= $connection_string . ':' . $settings['mongo_port'];
    }

    $server = 'mongodb://' . $connection_string;

    // Fire!
    try {

      $this->connection = new \MongoClient($server, $options);
    }
    catch (\Exception $e) {

      Controller::log('Could not connect to the Mongo database ' . $settings['mongo_host'] . '. Please check your connection settings.', 'error');
    }

    return $this;
  }

  /**
   * Use a database from the selected connection.
   *
   * @param string $db_name
   *   The database name.
   *
   * @return \MongoDB
   *   A PHP MongoDB object.
   */
  public function useDatabase($db_name) {
    if (!isset($this->connection) || empty($this->connection)) {

      Controller::log('Attempt to set database on an empty connection. Is MongoDB running?', 'warning');
      return FALSE;
    }

    try {

      $this->database = $this->connection->selectDB($db_name);
    }
    catch(\Exception $e) {

      Controller::log('Could not select database', 'error');
    }
    return $this->database;
  }
}
