<?php
/**
 * @file
 * Provides a Controller for Ming.
 *
 * @copyright Copyright(c) 2013 Previous Next Pty Ltd
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Chris Skene chris at previousnext dot com dot au
 */

namespace Drupal\ming\Ming;

/**
 * Class Controller
 *
 * @package ming\Ming
 */
class Controller {

  /**
   * Instantiate a ming database connection.
   *
   * If passing the $alias parameter, ming will instantiate a Mongo connection
   * on that alias, or reuse and existing one. This allows you to run concurrent
   * Mongo connections simultaneously. To switch databases within a connection,
   * use \Ming\Core\Database::setDatabase().
   *
   * @param string $alias
   *   The database to access. If none provided, returns that provided in config
   * @param array $override_settings
   *   An optional array of database settings. If provided, other connection
   *   settings are ignored.
   *
   * @return Database
   *   A Ming database object.
   */
  static public function connect($alias = MING_DEFAULT_CONNECTION_ALIAS, $override_settings = array()) {

    // Use the Ming connection.
    $ming = &drupal_static('ming_connection');

    if (isset($ming[$alias])) {
      return $ming[$alias];
    }
    else {
      if (self::useMongoDbModuleSettings()) {

        // Use the connection provided by the mongodb module.
        $mongodb_module_conn = mongodb($alias);

        // Mongodb assigns the connection to a non-existent property on the
        // MongoDB so we fix that by reassigning it within our own DB object.
        $connection = $mongodb_module_conn->connection;
        unset($mongodb_module_conn->connection);

        $database = new Database($mongodb_module_conn);
        $database->setConnection($connection);
      }
      else {

        // Use local settings, or collect from hooks.
        if (empty($override_settings)) {
          $settings = self::getSettings($alias);
        }
        else {
          $settings = $override_settings;
        }

        // Instantiate a connection object, pass it to our DB object.
        $connection = new Connection($settings);
        $mongodb_module_conn = $connection->useDatabase($settings['mongo_db']);

        $database = new Database($mongodb_module_conn);
        $database->connection = $connection->connection;
      }

      $ming[$alias] = $database;
    }

    return $ming[$alias];
  }

  /**
   * Load the ming connection settings.
   *
   * @param string $alias
   *   The connection alias to use.
   *
   * @return array|bool
   *   The array of connection settings, or FALSE.
   */
  static public function getSettings($alias = '_default') {

    static $settings = array();
    static $loaded;

    if (isset($loaded) && $loaded == TRUE) {
      if (isset($settings[$alias])) {
        return $settings[$alias];
      }
    }
    else {
      // Load defaults.
      // Invokes 'hook_ming_settings'.
      $settings = module_invoke_all('ming_settings');

      // Load local settings.
      $locals = variable_get('ming_settings', array());
      if (!empty($locals)) {
        $settings = $settings + $locals;
      }

      $loaded = TRUE;
    }

    // Return requested settings.
    if (isset($settings[$alias])) {
      return $settings[$alias];
    }

    return FALSE;
  }

  /**
   * Error handling.
   *
   * @param string $message
   *   The error message to set
   * @param string $type
   *   The type of message (corresponds to Drupals default types)
   * @param array $args
   *   Any arguments to substitute
   */
  static public function log($message, $type, $args = array()) {

    $e_level = variable_get('ming_error_handling', MING_ERROR_VERBOSE);

    // Log only.
    if ($e_level == MING_ERROR_LOG) {
      if (in_array($type, array('warning', 'error'))) {
        watchdog('ming', $message, $args);
      }
    }

    // Default errors.
    if ($e_level == MING_ERROR_NORMAL) {
      if (in_array($type, array('warning', 'error'))) {
        watchdog('ming', $message, $args);
        drupal_set_message($message, $type);
      }
    }

    // Verbose errors.
    if ($e_level == MING_ERROR_VERBOSE) {
      watchdog('ming', $message, $args);
      drupal_set_message($message, $type);
    }
  }

  /**
   * Should we use the MongoDB module settings.
   *
   * @return bool
   *   TRUE, if we should use the MongoDB module settings, or FALSE.
   */
  public static function useMongoDbModuleSettings() {

    if (variable_get('ming_use_mongodb_module', FALSE) && function_exists('mongodb')) {
      return TRUE;
    }

    return FALSE;
  }
}
