<?php
/**
 * @file
 * The database connection and tools.
 */

namespace Drupal\ming\Ming;

use MongoDB;
use MongoCollection;
use MongoCursor;
use MongoId;

/**
 * The Database connection and tools
 */
class Database {

  /**
   * The MongoDB object
   *
   * @var MongoDB
   */
  public $mongoDB;

  /**
   * The Mongo connection object
   */
  public $connection;

  /**
   * The currently active collection
   *
   * @var MongoCollection
   */
  public $collection;

  /**
   * The last inserted QID
   */
  protected $lastInsert;

  /**
   * The last retrieved find result
   */
  protected $lastCursor;

  /**
   * Constructor function.
   *
   * @param \MongoDB $mongo
   *   A native PHP MongoDB object.
   */
  public function __construct(MongoDB $mongo) {
    $this->setDatabase($mongo);
  }

  /**
   * Select the database to use.
   *
   * @param \MongoDB $mongo
   *   A native PHP MongoDB object.
   */
  public function setDatabase(MongoDB $mongo) {
    $this->mongoDB = $mongo;
  }

  /**
   * Set the connection.
   *
   * @param Connection $connection
   *   A connection object.
   */
  public function setConnection($connection) {
    $this->connection = $connection;
  }

  /**
   * Returns a collection object from a db.
   *
   * @param string $collection_name
   *   The collection name.
   *
   * @return bool|MongoCollection
   *   A MongoCollection object or FALSE.
   */
  public function collection($collection_name) {
    if (!isset($this->mongoDB)) {
      return FALSE;
    }
    $this->collection = $this->mongoDB->selectCollection($collection_name);

    return $this->collection;
  }

  /**
   * Insert into a collection.
   *
   * @todo Support optional parameters fsync and timeout
   * @todo Error handling for return values
   *
   * @param array $data
   *   An array of data.
   * @param bool $safe
   *   Whether the insert should be "safe" or not.
   *
   * @return array
   *   An array of keys indicating the status of the insert result:
   *    'success' - TRUE or FALSE, depending on the insert result. Note that
   *                non-safe queries may always return TRUE regardless of their
   *                actual success.
   *    'id' - The ID of the inserted object.
   *   If $safe is TRUE, the following Mongo keys will also be available:
   *    'n' - The number of objects affected.
   *    'connectionId'  - The connection in use
   *    'err' - A description of any error
   *    'ok'
   *
   * @link http://php.net/manual/en/mongocollection.insert.php
   */
  public function insert(array $data, $safe = FALSE) {
    if (empty($this->collection)) {
      return FALSE;
    }
    $options = array('safe' => $safe);
    $result = $this->collection->insert($data, $options);

    $this->processWriteResult($result, $data);

    return $this->lastInsert;
  }

  /**
   * Update the first matched item.
   *
   * For updating properties where only one item should exist
   *
   * @param array $filter
   *   An array of keys to match on
   * @param array $data
   *   The data to pass to Mongo
   * @param bool $partial
   *   If TRUE, this is a partial update and Ming will only update the fields
   *   provided in $data. If FALSE, $data completely overwrites the object.
   * @param array $options
   *   Any other options to pass to Mongo
   *
   * @todo $options is not passed!
   *
   * @return array
   *   See \Mongo\Core\Database::insert() for possible values
   *
   * @see \Mongo\Core\Database::insert()
   */
  public function update(array $filter, array $data, $partial = FALSE, $options = array()) {
    if (empty($this->collection)) {
      return FALSE;
    }

    // Update single - this is the default behaviour, however the Mongo docs
    // suggest that this should be set explicitly for future-proofing
    $options = array('multiple' => FALSE);

    // Fix partial keys.
    if (isset($partial) && $partial == TRUE) {
      $data = array('$set' => $data);
    }

    $result = $this->collection->update($filter, $data);

    $this->processWriteResult($result, $data);

    return $this->lastInsert;
  }

  /**
   * Update all matched items.
   *
   * Useful for changing properties across a range of items simultaneously.
   *
   * @todo Add support for atomic updates
   * @todo $partial is not respected
   *
   * @param array $filter
   *   An array of keys to match on
   * @param array $data
   *   The data to pass to Mongo
   * @param bool $partial
   *   If TRUE, this is a partial update and Ming will only update the fields
   *   provided in $data. If FALSE, $data completely overwrites the object.
   * @param array $options
   *   Any other options to pass to Mongo
   *
   * @return array
   *   See \Mongo\Core\Database::insert() for possible values
   *
   * @see \Mongo\Core\Database::insert()
   */
  public function updateAll(array $filter, array $data, $partial = FALSE, $options = array()) {
    if (empty($this->collection)) {
      return FALSE;
    }

    // Update multiple.
    $options['multiple'] = TRUE;

    $result = $this->collection->update($filter, $data, $options);

    $this->processWriteResult($result, $data);

    return $this->lastInsert;
  }

  /**
   * Update an individual item by ID.
   *
   * For updating properties on an item when you already know the item's $id
   *
   * @param string $identifier
   *   The ID to update.
   * @param array $data
   *   The data to update.
   *
   * @return array
   *   See \Mongo\Core\Database::insert() for possible values
   *
   * @see \Mongo\Core\Database::insert()
   */
  public function updateByID($identifier, array $data) {
    if (empty($this->collection)) {
      return FALSE;
    }

    $filter = $this->filterID($identifier);

    $result = $this->collection->update($filter, $data);

    $this->processWriteResult($result, $data);

    return $this->lastInsert;
  }

  /**
   * Upsert an item.
   *
   * For updating or creating an object if it does not already exist
   *
   * @param array $filter
   *   A normal MongoDB filter array
   * @param array $data
   *   The data to insert
   * @param bool $safe
   *   If TRUE, conduct a safe insert. Note that this returns an exception on
   *   error, which you will need to catch.
   *
   * @return array
   *   See \Mongo\Core\Database::insert() for possible values
   *
   * @see \Mongo\Core\Database::insert()
   */
  public function upsert($filter, array $data, $safe = FALSE) {
    if (empty($this->collection)) {
      return FALSE;
    }

    $options = array("upsert" => TRUE, "multiple" => FALSE, "safe" => $safe);

    $result = $this->collection->update($filter, $data, $options);

    $this->processWriteResult($result, $data);

    return $this->lastInsert;
  }

  /**
   * Delete items matching a filter.
   *
   * @param array $filter
   *   A normal MongoDB filter array
   * @param bool $safe
   *   If TRUE, conduct a safe insert. Note that this returns an exception on
   *   error, which you will need to catch.
   *
   * @return mixed
   *   Result of the Remove operation.
   */
  public function delete($filter, $safe = FALSE) {
    return $this->collection->remove($filter, array("safe" => $safe));
  }

  /**
   * Delete an item.
   *
   * @param string $identifier
   *   The ID of the item to delete
   * @param bool $safe
   *   If TRUE, conduct a safe insert. Note that this returns an exception on
   *   error, which you will need to catch.
   *
   * @return mixed
   *   Result of the Remove operation.
   */
  public function deleteByID($identifier, $safe = FALSE) {
    $filter = $this->filterID($identifier);
    return $this->collection->remove($filter, array("justOne" => TRUE, "safe" => $safe));
  }

  /**
   * Query the current collection.
   *
   * Simply a wrapper around MongoCollection::find(), this function returns a
   * MongoCursor object which should be foreach()ed to get its results. We
   * also store the returned value in $last_cursor just in case we need it
   * again.
   *
   * @param array $filter
   *   A standard MongoDB filter
   *
   * @return MongoCursor|bool
   *   A MongoCursor, or FALSE.
   */
  public function find(array $filter) {
    if (empty($this->collection)) {
      return FALSE;
    }

    $this->lastCursor = $this->collection->find($filter);

    return $this->lastCursor;
  }

  /**
   * Query by ID.
   *
   * As finding by ID should only return one result, we shortcut any cursor
   * foreach handling and simply translate the result into an array on return.
   *
   * @param string $identifier
   *   An ID of a Mongo document. We convert to a MongoID automatically.
   *
   * @return array|bool
   *   Either the result of the find as an array, or FALSE
   */
  public function findByID($identifier) {

    if (empty($this->collection)) {
      return FALSE;
    }

    $filter = $this->filterID($identifier);
    $this->lastCursor = $this->collection->find($filter);

    return iterator_to_array($this->lastCursor);
  }

  /**
   * Shortcut to return all results for a query in an array.
   *
   * @param array $filter
   *   A standard MongoDB filter
   *
   * @return array
   *   An array of results
   */
  public function findAll(array $filter) {
    $results = array();

    $this->find($filter);
    if ($this->lastCursor->hasNext()) {
      foreach ($this->lastCursor as $result) {
        $results[] = $result;
      }
    }

    return $results;
  }
  /**
   * Sanitize user input before using it in a filter.
   *
   * To avoid request injection attacks, user data used in queries should be in
   * the form of strings before sending it to the query functions (like find(),
   * for example). This function can be used to quickly sanitize data before
   * use.
   *
   * @param mixed $data
   *   The data to sanitize
   *
   * @return string
   *   A string.
   */
  public function sanitize($data) {
    return (string) $data;
  }

  /**
   * Shortcut to set up a filter for filtering by ID.
   *
   * @param string $identifier
   *   An identifier.
   *
   * @return array
   *   The Mongo filter.
   */
  public function filterID($identifier) {
    $mid = new MongoId($identifier);
    $filter = array('_id' => $mid);

    return $filter;
  }

  /**
   * Given a MongoID, return the ID number.
   *
   * @param array $data
   *   An array containing a MongoID.
   *
   * @return string|bool
   *   A string ID, or FALSE.
   */
  public function extractID($data) {
    $item = NULL;
    if (is_array($data) && isset($data['_id'])) {
      $item = (array) $data['_id'];
    }
    elseif (is_object($data)) {
      $item = (array) $data;
    }
    else {
      return FALSE;
    }

    if (isset($item['$id'])) {
      return $item['$id'];
    }

    return FALSE;
  }

  /**
   * Helper to add a modifier to data.
   *
   * @todo Individual validation for modifier types
   *
   * @param string $modifier
   *   The Mongo modifier to set
   * @param array $data
   *   The data to use
   *
   * @return array
   *   An array suitable for insertion in to Mongo
   */
  public function mod($modifier, $data) {
    if (is_array($data)) {
      foreach (array_keys($data) as $key) {
        if ($key == '_id') {
          unset($data[$key]);
          Controller::log('Mongo IDs can not be used in modifier operations', 'warning');
        }
      }
    }
    return array('$' . $modifier => $data);
  }

  /**
   * Process the result of a write operation.
   *
   * @param mixed $result
   *   The result returned by the write operation
   * @param mixed $data
   *   The data passed to the write operation
   */
  protected function processWriteResult($result, $data) {

    if (is_array($result)) {
      $this->lastInsert = $result;
      if (is_null($result['err'])) {
        $this->lastInsert['success'] = TRUE;
      }
      else {
        $this->lastInsert['success'] = FALSE;
      }
    }
    else {
      $this->lastInsert['success'] = $result;
    }

    $this->lastInsert['id'] = $this->extractID($data);
  }
}
