<?php
/**
 * @file
 * A base class for Items to be managed in and out of the Ming system.
 */

namespace Drupal\ming\Ming;

/**
 * Base class for an Item
 */
class Item {
  /**
   * Mongo ID
   *
   * @var string
   */
  protected $mid;

  /**
   * Properties to be written to the database
   *
   * @var array
   */
  protected $properties;

  /**
   * Collection to use
   *
   * @var string
   */
  protected $collection;

  /**
   * The active database connection
   *
   * @var \Drupal\ming\Ming\Database
   */
  protected $database;

  /**
   * Constructor.
   */
  public function __construct() {
    // Nothing to do currently.
  }

  /**
   * Set mid.
   *
   * This will check whether the provided ID is a MongoId object before setting
   * the value.
   *
   * @param string $identifier
   *   A suitable MongoDB identifier.
   */
  public function setIdentifier($identifier) {
    if (is_object($identifier) && get_class($identifier) == 'MongoId') {
      $this->mid = $identifier['$id'];
    }
    else {
      $this->mid = $identifier;
    }
  }

  /**
   * Set a property.
   *
   * @param string $key
   *   The property key.
   * @param mixed $value
   *   The value.
   */
  public function set($key, $value) {
    $this->properties[$key] = $value;
  }

  /**
   * Retrieve a property.
   *
   * @param string $key
   *   The property to return
   */
  public function get($key) {
    if (isset($this->properties[$key])) {
      return $this->properties[$key];
    }
  }

  /**
   * Unset a property.
   *
   * @param string $key
   *   Property to unset.
   */
  public function remove($key) {
    if (isset($this->properties[$key])) {
      unset($this->properties[$key]);
    }
  }

  /**
   * Save the current item to the database.
   *
   * This will either create or update an existing item, using the Mongo upsert
   * functionality. It also sets time created (if new) and time updated.
   *
   * @param string|null $alias
   *   The connection alias to use
   *
   * @return array
   *   Result of the Upser.
   */
  public function save($alias = NULL) {

    // Set default created/updated properties.
    if (!isset($this->properties['time_created'])) {
      $this->set('time_created', time());
    }
    $this->set('time_updated', time());

    // Load a database.
    $this->database($alias);

    if (isset($this->mid)) {
      $filter = $this->database->filterID($this->mid);
    }
    else {
      $filter = array();
    }

    return $this->database->upsert($filter, $this->items());
  }

  /**
   * Access a Ming DB connection.
   *
   * @param string $alias
   *   The alias for the connection to use. If none is set, the default is used.
   * @param bool|string $reset
   *   If TRUE, reload the connection from source, otherwise, use an existing
   *   connection if available. Note, this only reattaches the Ming database
   *   currently in use to THIS object, it has no effect on the connection
   *   itself or its persistence.
   * @param array $overrides
   *   An optional array of database settings. If provided, other connection
   *   settings are ignored.
   *
   * @return \Drupal\ming\Ming\Database
   *   A Ming Database object.
   */
  public function database($alias = MING_DEFAULT_CONNECTION_ALIAS, $reset = FALSE, $overrides = array()) {
    if (!isset($this->database) || empty($this->database) || $reset == TRUE) {
      $this->database = Controller::connect($alias, $overrides);
    }
    $this->database->collection($this->collection());

    return $this->database;
  }

  /**
   * Set the collection to use for this item.
   *
   * A value can be passed to this function to set the active collection, or
   * the method can be overridden and simply return a default.
   *
   * @param string $collection_name
   *   [optional] If provided, the collection in use will be set to the string
   *   provided.
   *
   * @return string|bool
   *   If the collection name is set, returns the collection name, otherwise
   *   FALSE.
   */
  public function setCollection($collection_name = NULL) {
    if (!empty($collection_name)) {
      $this->collection = $collection_name;
    }
    return $this->collection();
  }

  /**
   * Returns the current collection name.
   *
   * @return string|bool
   *   Either the name of the collection, or FALSE.
   */
  public function collection() {
    if (!empty($this->collection)) {
      return $this->collection;
    }

    return FALSE;
  }

  /**
   * Return an objects properties as an array for inserting into Mongo.
   *
   * @return array
   *   An array of properties.
   */
  public function items() {
    return $this->properties;
  }
}
